@@ -5,7 +5,6 @@ Hexbin plot with marginal distributions
 _thumb: .45, .4
 """
 import numpy as np
-from scipy.stats import kendalltau
 import seaborn as sns
 sns.set(style="ticks")
 
@@ -13,4 +12,4 @@ rs = np.random.RandomState(11)
 x = rs.gamma(2, size=1000)
 y = -.5 * x + rs.normal(size=1000)
 
-sns.jointplot(x, y, kind="hex", stat_func=kendalltau, color="#4CB391")
+sns.jointplot(x, y, kind="hex", color="#4CB391")
