@@ -7,12 +7,12 @@ _thumb: .45, .45
 import seaborn as sns
 sns.set()
 
-# Load the example tips dataset
+# Load the iris dataset
 iris = sns.load_dataset("iris")
 
-# Plot tip as a function of toal bill across days
+# Plot sepal with as a function of sepal_length across days
 g = sns.lmplot(x="sepal_length", y="sepal_width", hue="species",
-               truncate=True, size=5, data=iris)
+               truncate=True, height=5, data=iris)
 
 # Use more informative axis labels than are provided by default
 g.set_axis_labels("Sepal length (mm)", "Sepal width (mm)")
