@@ -1,16 +1,15 @@
 """
-Paired Density and Scatterplot Matrix
+Paired density and scatterplot matrix
 =====================================
 
 _thumb: .5, .5
 """
 import seaborn as sns
-import matplotlib.pyplot as plt
 sns.set(style="white")
 
 df = sns.load_dataset("iris")
 
 g = sns.PairGrid(df, diag_sharey=False)
-g.map_lower(sns.kdeplot, cmap="Blues_d")
-g.map_upper(plt.scatter)
+g.map_lower(sns.kdeplot)
+g.map_upper(sns.scatterplot)
 g.map_diag(sns.kdeplot, lw=3)
