@@ -2,11 +2,11 @@
 Linear regression with marginal distributions
 =============================================
 
-_thumb: .5, .6
+_thumb: .65, .65
 """
 import seaborn as sns
-sns.set(style="darkgrid", color_codes=True)
+sns.set(style="darkgrid")
 
 tips = sns.load_dataset("tips")
 g = sns.jointplot("total_bill", "tip", data=tips, kind="reg",
-                  xlim=(0, 60), ylim=(0, 12), color="r", size=7)
+                  xlim=(0, 60), ylim=(0, 12), color="m", height=7)
