@@ -4,35 +4,61 @@ from itertools import cycle
 
 import numpy as np
 import matplotlib as mpl
-import matplotlib.pyplot as plt
-from matplotlib.colors import LinearSegmentedColormap
 
 from .external import husl
 from .external.six import string_types
 from .external.six.moves import range
 
-from .utils import desaturate, set_hls_values
-from .xkcd_rgb import xkcd_rgb
-from .crayons import crayons
-from .miscplot import palplot
+from .utils import desaturate, set_hls_values, get_color_cycle
+from .colors import xkcd_rgb, crayons
+
+
+__all__ = ["color_palette", "hls_palette", "husl_palette", "mpl_palette",
+           "dark_palette", "light_palette", "diverging_palette",
+           "blend_palette", "xkcd_palette", "crayon_palette",
+           "cubehelix_palette", "set_color_codes"]
 
 
 SEABORN_PALETTES = dict(
-    deep=["#4C72B0", "#55A868", "#C44E52",
-          "#8172B2", "#CCB974", "#64B5CD"],
-    muted=["#4878CF", "#6ACC65", "#D65F5F",
-           "#B47CC7", "#C4AD66", "#77BEDB"],
-    pastel=["#92C6FF", "#97F0AA", "#FF9F9A",
-            "#D0BBFF", "#FFFEA3", "#B0E0E6"],
-    bright=["#003FFF", "#03ED3A", "#E8000B",
-            "#8A2BE2", "#FFC400", "#00D7FF"],
-    dark=["#001C7F", "#017517", "#8C0900",
-          "#7600A1", "#B8860B", "#006374"],
-    colorblind=["#0072B2", "#009E73", "#D55E00",
-                "#CC79A7", "#F0E442", "#56B4E9"]
+    deep=["#4C72B0", "#DD8452", "#55A868", "#C44E52", "#8172B3",
+          "#937860", "#DA8BC3", "#8C8C8C", "#CCB974", "#64B5CD"],
+    deep6=["#4C72B0", "#55A868", "#C44E52",
+           "#8172B3", "#CCB974", "#64B5CD"],
+    muted=["#4878D0", "#EE854A", "#6ACC64", "#D65F5F", "#956CB4",
+           "#8C613C", "#DC7EC0", "#797979", "#D5BB67", "#82C6E2"],
+    muted6=["#4878D0", "#6ACC64", "#D65F5F",
+            "#956CB4", "#D5BB67", "#82C6E2"],
+    pastel=["#A1C9F4", "#FFB482", "#8DE5A1", "#FF9F9B", "#D0BBFF",
+            "#DEBB9B", "#FAB0E4", "#CFCFCF", "#FFFEA3", "#B9F2F0"],
+    pastel6=["#A1C9F4", "#8DE5A1", "#FF9F9B",
+             "#D0BBFF", "#FFFEA3", "#B9F2F0"],
+    bright=["#023EFF", "#FF7C00", "#1AC938", "#E8000B", "#8B2BE2",
+            "#9F4800", "#F14CC1", "#A3A3A3", "#FFC400", "#00D7FF"],
+    bright6=["#023EFF", "#1AC938", "#E8000B",
+             "#8B2BE2", "#FFC400", "#00D7FF"],
+    dark=["#001C7F", "#B1400D", "#12711C", "#8C0800", "#591E71",
+          "#592F0D", "#A23582", "#3C3C3C", "#B8850A", "#006374"],
+    dark6=["#001C7F", "#12711C", "#8C0800",
+           "#591E71", "#B8850A", "#006374"],
+    colorblind=["#0173B2", "#DE8F05", "#029E73", "#D55E00", "#CC78BC",
+                "#CA9161", "#FBAFE4", "#949494", "#ECE133", "#56B4E9"],
+    colorblind6=["#0173B2", "#029E73", "#D55E00",
+                 "#CC78BC", "#ECE133", "#56B4E9"]
     )
 
 
+MPL_QUAL_PALS = {
+    "tab10": 10, "tab20": 20, "tab20b": 20, "tab20c": 20,
+    "Set1": 9, "Set2": 8, "Set3": 12,
+    "Accent": 8, "Paired": 12,
+    "Pastel1": 9, "Pastel2": 8, "Dark2": 8,
+}
+
+
+QUAL_PALETTE_SIZES = MPL_QUAL_PALS.copy()
+QUAL_PALETTE_SIZES.update({k: len(v) for k, v in SEABORN_PALETTES.items()})
+
+
 class _ColorPalette(list):
     """Set the color palette in a with statement, otherwise be a list."""
     def __enter__(self):
@@ -56,23 +82,26 @@ class _ColorPalette(list):
 def color_palette(palette=None, n_colors=None, desat=None):
     """Return a list of colors defining a color palette.
 
-    Availible seaborn palette names:
+    Available seaborn palette names:
         deep, muted, bright, pastel, dark, colorblind
 
     Other options:
-        hls, husl, any named matplotlib palette, list of colors
+        name of matplotlib cmap, 'ch:<cubehelix arguments>', 'hls', 'husl',
+        or a list of colors in any format matplotlib accepts
 
     Calling this function with ``palette=None`` will return the current
     matplotlib color cycle.
 
-    Matplotlib paletes can be specified as reversed palettes by appending
-    "_r" to the name or as dark palettes by appending "_d" to the name.
+    Matplotlib palettes can be specified as reversed palettes by appending
+    "_r" to the name or as "dark" palettes by appending "_d" to the name.
     (These options are mutually exclusive, but the resulting list of colors
     can also be reversed).
 
     This function can also be used in a ``with`` statement to temporarily
     set the color cycle for a plot or set of plots.
 
+    See the :ref:`tutorial <palette_tutorial>` for more information.
+
     Parameters
     ----------
     palette: None, string, or sequence, optional
@@ -103,40 +132,54 @@ def color_palette(palette=None, n_colors=None, desat=None):
     Examples
     --------
 
-    Show one of the "seaborn palettes", which have the same basic order of hues
-    as the default matplotlib color cycle but more attractive colors.
+    Calling with no arguments returns all colors from the current default
+    color cycle:
 
     .. plot::
         :context: close-figs
 
         >>> import seaborn as sns; sns.set()
+        >>> sns.palplot(sns.color_palette())
+
+    Show one of the other "seaborn palettes", which have the same basic order
+    of hues as the default matplotlib color cycle but more attractive colors.
+    Calling with the name of a palette will return 6 colors by default:
+
+    .. plot::
+        :context: close-figs
+
         >>> sns.palplot(sns.color_palette("muted"))
 
-    Use discrete values from one of the built-in matplotlib colormaps.
+    Use discrete values from one of the built-in matplotlib colormaps:
 
     .. plot::
         :context: close-figs
 
         >>> sns.palplot(sns.color_palette("RdBu", n_colors=7))
 
-    Make a "dark" matplotlib sequential palette variant. (This can be good
-    when coloring multiple lines or points that correspond to an ordered
-    variable, where you don't want the lightest lines to be invisible).
+    Make a customized cubehelix color palette:
 
     .. plot::
         :context: close-figs
 
-        >>> sns.palplot(sns.color_palette("Blues_d"))
+        >>> sns.palplot(sns.color_palette("ch:2.5,-.2,dark=.3"))
 
-    Use a categorical matplotlib palette, add some desaturation. (This can be
-    good when making plots with large patches, which look best with dimmer
-    colors).
+    Use a categorical matplotlib palette and add some desaturation:
 
     .. plot::
         :context: close-figs
 
         >>> sns.palplot(sns.color_palette("Set1", n_colors=8, desat=.5))
 
+    Make a "dark" matplotlib sequential palette variant. (This can be good
+    when coloring multiple lines or points that correspond to an ordered
+    variable, where you don't want the lightest lines to be invisible):
+
+    .. plot::
+        :context: close-figs
+
+        >>> sns.palplot(sns.color_palette("Blues_d"))
+
     Use as a context manager:
 
     .. plot::
@@ -148,7 +191,7 @@ def color_palette(palette=None, n_colors=None, desat=None):
 
     """
     if palette is None:
-        palette = mpl.rcParams["axes.color_cycle"]
+        palette = get_color_cycle()
         if n_colors is None:
             n_colors = len(palette)
 
@@ -159,22 +202,36 @@ def color_palette(palette=None, n_colors=None, desat=None):
     else:
 
         if n_colors is None:
-            n_colors = 6
+            # Use all colors in a qualitative palette or 6 of another kind
+            n_colors = QUAL_PALETTE_SIZES.get(palette, 6)
 
-        if palette == "hls":
-            palette = hls_palette(n_colors)
-        elif palette == "husl":
-            palette = husl_palette(n_colors)
-        elif palette.lower() == "jet":
-            raise ValueError("No.")
-        elif palette in SEABORN_PALETTES:
+        if palette in SEABORN_PALETTES:
+            # Named "seaborn variant" of old matplotlib default palette
             palette = SEABORN_PALETTES[palette]
-        elif palette in dir(mpl.cm):
-            palette = mpl_palette(palette, n_colors)
-        elif palette[:-2] in dir(mpl.cm):
-            palette = mpl_palette(palette, n_colors)
+
+        elif palette == "hls":
+            # Evenly spaced colors in cylindrical RGB space
+            palette = hls_palette(n_colors)
+
+        elif palette == "husl":
+            # Evenly spaced colors in cylindrical Lab space
+            palette = husl_palette(n_colors)
+
+        elif palette.lower() == "jet":
+            # Paternalism
+            raise ValueError("No.")
+
+        elif palette.startswith("ch:"):
+            # Cubehelix palette with params specified in string
+            args, kwargs = _parse_cubehelix_args(palette)
+            palette = cubehelix_palette(n_colors, *args, **kwargs)
+
         else:
-            raise ValueError("%s is not a valid palette name" % palette)
+            try:
+                # Perhaps a named matplotlib colormap?
+                palette = mpl_palette(palette, n_colors)
+            except ValueError:
+                raise ValueError("%s is not a valid palette name" % palette)
 
     if desat is not None:
         palette = [desaturate(c, desat) for c in palette]
@@ -193,7 +250,7 @@ def color_palette(palette=None, n_colors=None, desat=None):
     return palette
 
 
-def hls_palette(n_colors=6, h=.01, l=.6, s=.65):
+def hls_palette(n_colors=6, h=.01, l=.6, s=.65):  # noqa
     """Get a set of evenly spaced colors in HLS hue space.
 
     h, l, and s should be between 0 and 1
@@ -261,7 +318,7 @@ def hls_palette(n_colors=6, h=.01, l=.6, s=.65):
     return _ColorPalette(palette)
 
 
-def husl_palette(n_colors=6, h=.01, s=.9, l=.65):
+def husl_palette(n_colors=6, h=.01, s=.9, l=.65):  # noqa
     """Get a set of evenly spaced colors in HUSL hue space.
 
     h, s, and l should be between 0 and 1
@@ -326,7 +383,7 @@ def husl_palette(n_colors=6, h=.01, s=.9, l=.65):
     hues %= 1
     hues *= 359
     s *= 99
-    l *= 99
+    l *= 99  # noqa
     palette = [husl.husl_to_rgb(h_i, s, l) for h_i in hues]
     return _ColorPalette(palette)
 
@@ -391,18 +448,16 @@ def mpl_palette(name, n_colors=6):
         >>> sns.palplot(sns.mpl_palette("GnBu_d"))
 
     """
-    brewer_qual_pals = {"Accent": 8, "Dark2": 8, "Paired": 12,
-                        "Pastel1": 9, "Pastel2": 8,
-                        "Set1": 9, "Set2": 8, "Set3": 12}
-
     if name.endswith("_d"):
         pal = ["#333333"]
         pal.extend(color_palette(name.replace("_d", "_r"), 2))
         cmap = blend_palette(pal, n_colors, as_cmap=True)
     else:
-        cmap = getattr(mpl.cm, name)
-    if name in brewer_qual_pals:
-        bins = np.linspace(0, 1, brewer_qual_pals[name])[:n_colors]
+        cmap = mpl.cm.get_cmap(name)
+        if cmap is None:
+            raise ValueError("{} is not a valid colormap".format(name))
+    if name in MPL_QUAL_PALS:
+        bins = np.linspace(0, 1, MPL_QUAL_PALS[name])[:n_colors]
     else:
         bins = np.linspace(0, 1, n_colors + 2)[1:-1]
     palette = list(map(tuple, cmap(bins)[:, :3]))
@@ -581,7 +636,7 @@ def light_palette(color, n_colors=6, reverse=False, as_cmap=False,
 
     """
     color = _color_to_rgb(color, input)
-    light = set_hls_values(color, l=.95)
+    light = set_hls_values(color, l=.95)  # noqa
     colors = [color, light] if reverse else [light, color]
     return blend_palette(colors, n_colors, as_cmap)
 
@@ -613,8 +668,8 @@ def _flat_palette(color, n_colors=6, reverse=False, as_cmap=False,
     return blend_palette(colors, n_colors, as_cmap)
 
 
-def diverging_palette(h_neg, h_pos, s=75, l=50, sep=10, n=6, center="light",
-                      as_cmap=False):
+def diverging_palette(h_neg, h_pos, s=75, l=50, sep=10, n=6,  # noqa
+                      center="light", as_cmap=False):
     """Make a diverging palette between two HUSL colors.
 
     If you are using the IPython notebook, you can also choose this palette
@@ -726,7 +781,7 @@ def blend_palette(colors, n_colors=6, as_cmap=False, input="rgb"):
 def xkcd_palette(colors):
     """Make a palette with color names from the xkcd color survey.
 
-    See xkcd for the full list of colors: http://xkcd.com/color/rgb/
+    See xkcd for the full list of colors: https://xkcd.com/color/rgb/
 
     This is just a simple wrapper around the ``seaborn.xkcd_rgb`` dictionary.
 
@@ -754,7 +809,7 @@ def crayon_palette(colors):
     """Make a palette with color names from Crayola crayons.
 
     Colors are taken from here:
-    http://en.wikipedia.org/wiki/List_of_Crayola_crayon_colors
+    https://en.wikipedia.org/wiki/List_of_Crayola_crayon_colors
 
     This is just a simple wrapper around the ``seaborn.crayons`` dictionary.
 
@@ -785,10 +840,14 @@ def cubehelix_palette(n_colors=6, start=0, rot=.4, gamma=1.0, hue=0.8,
     This produces a colormap with linearly-decreasing (or increasing)
     brightness. That means that information will be preserved if printed to
     black and white or viewed by someone who is colorblind.  "cubehelix" is
-    also availible as a matplotlib-based palette, but this function gives the
+    also available as a matplotlib-based palette, but this function gives the
     user more control over the look of the palette and has a different set of
     defaults.
 
+    In addition to using this function, it is also possible to generate a
+    cubehelix palette generally in seaborn using a string-shorthand; see the
+    example below.
+
     Parameters
     ----------
     n_colors : int
@@ -881,8 +940,36 @@ def cubehelix_palette(n_colors=6, start=0, rot=.4, gamma=1.0, hue=0.8,
         >>> cmap = sns.cubehelix_palette(dark=0, light=1, as_cmap=True)
         >>> ax = sns.heatmap(x, cmap=cmap)
 
+    Use through the :func:`color_palette` interface:
+
+    .. plot::
+        :context: close-figs
+
+        >>> sns.palplot(sns.color_palette("ch:2,r=.2,l=.6"))
+
     """
-    cdict = mpl._cm.cubehelix(gamma, start, rot, hue)
+    def get_color_function(p0, p1):
+        # Copied from matplotlib because it lives in private module
+        def color(x):
+            # Apply gamma factor to emphasise low or high intensity values
+            xg = x ** gamma
+
+            # Calculate amplitude and angle of deviation from the black
+            # to white diagonal in the plane of constant
+            # perceived intensity.
+            a = hue * xg * (1 - xg) / 2
+
+            phi = 2 * np.pi * (start / 3 + rot * x)
+
+            return xg + a * (p0 * np.cos(phi) + p1 * np.sin(phi))
+        return color
+
+    cdict = {
+            "red": get_color_function(-0.14861, 1.78277),
+            "green": get_color_function(-0.29227, -0.90649),
+            "blue": get_color_function(1.97294, 0.0),
+    }
+
     cmap = mpl.colors.LinearSegmentedColormap("cubehelix", cdict)
 
     x = np.linspace(light, dark, n_colors)
@@ -895,12 +982,47 @@ def cubehelix_palette(n_colors=6, start=0, rot=.4, gamma=1.0, hue=0.8,
         if reverse:
             x_256 = x_256[::-1]
         pal_256 = cmap(x_256)
-        cmap = mpl.colors.ListedColormap(pal_256)
+        cmap = mpl.colors.ListedColormap(pal_256, "seaborn_cubehelix")
         return cmap
     else:
         return _ColorPalette(pal)
 
 
+def _parse_cubehelix_args(argstr):
+    """Turn stringified cubehelix params into args/kwargs."""
+
+    if argstr.startswith("ch:"):
+        argstr = argstr[3:]
+
+    if argstr.endswith("_r"):
+        reverse = True
+        argstr = argstr[:-2]
+    else:
+        reverse = False
+
+    if not argstr:
+        return [], {"reverse": reverse}
+
+    all_args = argstr.split(",")
+
+    args = [float(a.strip(" ")) for a in all_args if "=" not in a]
+
+    kwargs = [a.split("=") for a in all_args if "=" in a]
+    kwargs = {k.strip(" "): float(v.strip(" ")) for k, v in kwargs}
+
+    kwarg_map = dict(
+        s="start", r="rot", g="gamma",
+        h="hue", l="light", d="dark",  # noqa: E741
+    )
+
+    kwargs = {kwarg_map.get(k, k): v for k, v in kwargs.items()}
+
+    if reverse:
+        kwargs["reverse"] = True
+
+    return args, kwargs
+
+
 def set_color_codes(palette="deep"):
     """Change how matplotlib color shorthands are interpreted.
 
@@ -945,425 +1067,15 @@ def set_color_codes(palette="deep"):
     if palette == "reset":
         colors = [(0., 0., 1.), (0., .5, 0.), (1., 0., 0.), (.75, .75, 0.),
                   (.75, .75, 0.), (0., .75, .75), (0., 0., 0.)]
-    else:
+    elif palette in SEABORN_PALETTES:
+        if not palette.endswith("6"):
+            palette = palette + "6"
         colors = SEABORN_PALETTES[palette] + [(.1, .1, .1)]
+    else:
+        err = "Cannot set colors with palette '{}'".format(palette)
+        raise ValueError(err)
+
     for code, color in zip("bgrmyck", colors):
         rgb = mpl.colors.colorConverter.to_rgb(color)
         mpl.colors.colorConverter.colors[code] = rgb
         mpl.colors.colorConverter.cache[code] = rgb
-
-
-def _init_mutable_colormap():
-    """Create a matplotlib colormap that will be updated by the widgets."""
-    greys = color_palette("Greys", 256)
-    cmap = LinearSegmentedColormap.from_list("interactive", greys)
-    cmap._init()
-    cmap._set_extremes()
-    return cmap
-
-
-def _update_lut(cmap, colors):
-    """Change the LUT values in a matplotlib colormap in-place."""
-    cmap._lut[:256] = colors
-    cmap._set_extremes()
-
-
-def _show_cmap(cmap):
-    """Show a continuous matplotlib colormap."""
-    from .rcmod import axes_style  # Avoid circular import
-    with axes_style("white"):
-        f, ax = plt.subplots(figsize=(8.25, .75))
-    ax.set(xticks=[], yticks=[])
-    x = np.linspace(0, 1, 256)[np.newaxis, :]
-    ax.pcolormesh(x, cmap=cmap)
-
-
-def choose_colorbrewer_palette(data_type, as_cmap=False):
-    """Select a palette from the ColorBrewer set.
-
-    These palettes are built into matplotlib and can be used by name in
-    many seaborn functions, or by passing the object returned by this function.
-
-    Parameters
-    ----------
-    data_type : {'sequential', 'diverging', 'qualitative'}
-        This describes the kind of data you want to visualize. See the seaborn
-        color palette docs for more information about how to choose this value.
-        Note that you can pass substrings (e.g. 'q' for 'qualitative.
-
-    as_cmap : bool
-        If True, the return value is a matplotlib colormap rather than a
-        list of discrete colors.
-
-    Returns
-    -------
-    pal or cmap : list of colors or matplotlib colormap
-        Object that can be passed to plotting functions.
-
-    See Also
-    --------
-    dark_palette : Create a sequential palette with dark low values.
-    light_palette : Create a sequential palette with bright low values.
-    diverging_palette : Create a diverging palette from selected colors.
-    cubehelix_palette : Create a sequential palette or colormap using the
-                        cubehelix system.
-
-
-    """
-    from IPython.html.widgets import interact, FloatSliderWidget
-
-    if data_type.startswith("q") and as_cmap:
-        raise ValueError("Qualitative palettes cannot be colormaps.")
-
-    pal = []
-    if as_cmap:
-        cmap = _init_mutable_colormap()
-
-    if data_type.startswith("s"):
-        opts = ["Greys", "Reds", "Greens", "Blues", "Oranges", "Purples",
-                "BuGn", "BuPu", "GnBu", "OrRd", "PuBu", "PuRd", "RdPu", "YlGn",
-                "PuBuGn", "YlGnBu", "YlOrBr", "YlOrRd"]
-        variants = ["regular", "reverse", "dark"]
-
-        @interact
-        def choose_sequential(name=opts, n=(2, 18),
-                              desat=FloatSliderWidget(min=0, max=1, value=1),
-                              variant=variants):
-            if variant == "reverse":
-                name += "_r"
-            elif variant == "dark":
-                name += "_d"
-
-            if as_cmap:
-                colors = color_palette(name, 256, desat)
-                _update_lut(cmap, np.c_[colors, np.ones(256)])
-                _show_cmap(cmap)
-            else:
-                pal[:] = color_palette(name, n, desat)
-                palplot(pal)
-
-    elif data_type.startswith("d"):
-        opts = ["RdBu", "RdGy", "PRGn", "PiYG", "BrBG",
-                "RdYlBu", "RdYlGn", "Spectral"]
-        variants = ["regular", "reverse"]
-
-        @interact
-        def choose_diverging(name=opts, n=(2, 16),
-                             desat=FloatSliderWidget(min=0, max=1, value=1),
-                             variant=variants):
-            if variant == "reverse":
-                name += "_r"
-            if as_cmap:
-                colors = color_palette(name, 256, desat)
-                _update_lut(cmap, np.c_[colors, np.ones(256)])
-                _show_cmap(cmap)
-            else:
-                pal[:] = color_palette(name, n, desat)
-                palplot(pal)
-
-    elif data_type.startswith("q"):
-        opts = ["Set1", "Set2", "Set3", "Paired", "Accent",
-                "Pastel1", "Pastel2", "Dark2"]
-
-        @interact
-        def choose_qualitative(name=opts, n=(2, 16),
-                               desat=FloatSliderWidget(min=0, max=1, value=1)):
-            pal[:] = color_palette(name, n, desat)
-            palplot(pal)
-
-    if as_cmap:
-        return cmap
-    return pal
-
-
-def choose_dark_palette(input="husl", as_cmap=False):
-    """Launch an interactive widget to create a dark sequential palette.
-
-    This corresponds with the :func:`dark_palette` function. This kind
-    of palette is good for data that range between relatively uninteresting
-    low values and interesting high values.
-
-    Requires IPython 2+ and must be used in the notebook.
-
-    Parameters
-    ----------
-    input : {'husl', 'hls', 'rgb'}
-        Color space for defining the seed value. Note that the default is
-        different than the default input for :func:`dark_palette`.
-    as_cmap : bool
-        If True, the return value is a matplotlib colormap rather than a
-        list of discrete colors.
-
-    Returns
-    -------
-    pal or cmap : list of colors or matplotlib colormap
-        Object that can be passed to plotting functions.
-
-    See Also
-    --------
-    dark_palette : Create a sequential palette with dark low values.
-    light_palette : Create a sequential palette with bright low values.
-    cubehelix_palette : Create a sequential palette or colormap using the
-                        cubehelix system.
-
-    """
-    from IPython.html.widgets import interact
-
-    pal = []
-    if as_cmap:
-        cmap = _init_mutable_colormap()
-
-    if input == "rgb":
-        @interact
-        def choose_dark_palette_rgb(r=(0., 1.),
-                                    g=(0., 1.),
-                                    b=(0., 1.),
-                                    n=(3, 17)):
-            color = r, g, b
-            if as_cmap:
-                colors = dark_palette(color, 256, input="rgb")
-                _update_lut(cmap, colors)
-                _show_cmap(cmap)
-            else:
-                pal[:] = dark_palette(color, n, input="rgb")
-                palplot(pal)
-
-    elif input == "hls":
-        @interact
-        def choose_dark_palette_hls(h=(0., 1.),
-                                    l=(0., 1.),
-                                    s=(0., 1.),
-                                    n=(3, 17)):
-            color = h, l, s
-            if as_cmap:
-                colors = dark_palette(color, 256, input="hls")
-                _update_lut(cmap, colors)
-                _show_cmap(cmap)
-            else:
-                pal[:] = dark_palette(color, n, input="hls")
-                palplot(pal)
-
-    elif input == "husl":
-        @interact
-        def choose_dark_palette_husl(h=(0, 359),
-                                     s=(0, 99),
-                                     l=(0, 99),
-                                     n=(3, 17)):
-            color = h, s, l
-            if as_cmap:
-                colors = dark_palette(color, 256, input="husl")
-                _update_lut(cmap, colors)
-                _show_cmap(cmap)
-            else:
-                pal[:] = dark_palette(color, n, input="husl")
-                palplot(pal)
-
-    if as_cmap:
-        return cmap
-    return pal
-
-
-def choose_light_palette(input="husl", as_cmap=False):
-    """Launch an interactive widget to create a light sequential palette.
-
-    This corresponds with the :func:`light_palette` function. This kind
-    of palette is good for data that range between relatively uninteresting
-    low values and interesting high values.
-
-    Requires IPython 2+ and must be used in the notebook.
-
-    Parameters
-    ----------
-    input : {'husl', 'hls', 'rgb'}
-        Color space for defining the seed value. Note that the default is
-        different than the default input for :func:`light_palette`.
-    as_cmap : bool
-        If True, the return value is a matplotlib colormap rather than a
-        list of discrete colors.
-
-    Returns
-    -------
-    pal or cmap : list of colors or matplotlib colormap
-        Object that can be passed to plotting functions.
-
-    See Also
-    --------
-    light_palette : Create a sequential palette with bright low values.
-    dark_palette : Create a sequential palette with dark low values.
-    cubehelix_palette : Create a sequential palette or colormap using the
-                        cubehelix system.
-
-    """
-    from IPython.html.widgets import interact
-
-    pal = []
-    if as_cmap:
-        cmap = _init_mutable_colormap()
-
-    if input == "rgb":
-        @interact
-        def choose_light_palette_rgb(r=(0., 1.),
-                                     g=(0., 1.),
-                                     b=(0., 1.),
-                                     n=(3, 17)):
-            color = r, g, b
-            if as_cmap:
-                colors = light_palette(color, 256, input="rgb")
-                _update_lut(cmap, colors)
-                _show_cmap(cmap)
-            else:
-                pal[:] = light_palette(color, n, input="rgb")
-                palplot(pal)
-
-    elif input == "hls":
-        @interact
-        def choose_light_palette_hls(h=(0., 1.),
-                                     l=(0., 1.),
-                                     s=(0., 1.),
-                                     n=(3, 17)):
-            color = h, l, s
-            if as_cmap:
-                colors = light_palette(color, 256, input="hls")
-                _update_lut(cmap, colors)
-                _show_cmap(cmap)
-            else:
-                pal[:] = light_palette(color, n, input="hls")
-                palplot(pal)
-
-    elif input == "husl":
-        @interact
-        def choose_light_palette_husl(h=(0, 359),
-                                      s=(0, 99),
-                                      l=(0, 99),
-                                      n=(3, 17)):
-            color = h, s, l
-            if as_cmap:
-                colors = light_palette(color, 256, input="husl")
-                _update_lut(cmap, colors)
-                _show_cmap(cmap)
-            else:
-                pal[:] = light_palette(color, n, input="husl")
-                palplot(pal)
-
-    if as_cmap:
-        return cmap
-    return pal
-
-
-def choose_diverging_palette(as_cmap=False):
-    """Launch an interactive widget to choose a diverging color palette.
-
-    This corresponds with the :func:`diverging_palette` function. This kind
-    of palette is good for data that range between interesting low values
-    and interesting high values with a meaningful midpoint. (For example,
-    change scores relative to some baseline value).
-
-    Requires IPython 2+ and must be used in the notebook.
-
-    Parameters
-    ----------
-    as_cmap : bool
-        If True, the return value is a matplotlib colormap rather than a
-        list of discrete colors.
-
-    Returns
-    -------
-    pal or cmap : list of colors or matplotlib colormap
-        Object that can be passed to plotting functions.
-
-    See Also
-    --------
-    diverging_palette : Create a diverging color palette or colormap.
-    choose_colorbrewer_palette : Interactively choose palettes from the
-                                 colorbrewer set, including diverging palettes.
-
-    """
-    from IPython.html.widgets import interact, IntSliderWidget
-
-    pal = []
-    if as_cmap:
-        cmap = _init_mutable_colormap()
-
-    @interact
-    def choose_diverging_palette(h_neg=IntSliderWidget(min=0,
-                                                       max=359,
-                                                       value=220),
-                                 h_pos=IntSliderWidget(min=0,
-                                                       max=359,
-                                                       value=10),
-                                 s=IntSliderWidget(min=0, max=99, value=74),
-                                 l=IntSliderWidget(min=0, max=99, value=50),
-                                 sep=IntSliderWidget(min=1, max=50, value=10),
-                                 n=(2, 16),
-                                 center=["light", "dark"]):
-        if as_cmap:
-            colors = diverging_palette(h_neg, h_pos, s, l, sep, 256, center)
-            _update_lut(cmap, colors)
-            _show_cmap(cmap)
-        else:
-            pal[:] = diverging_palette(h_neg, h_pos, s, l, sep, n, center)
-            palplot(pal)
-
-    if as_cmap:
-        return cmap
-    return pal
-
-
-def choose_cubehelix_palette(as_cmap=False):
-    """Launch an interactive widget to create a sequential cubehelix palette.
-
-    This corresponds with the :func:`cubehelix_palette` function. This kind
-    of palette is good for data that range between relatively uninteresting
-    low values and interesting high values. The cubehelix system allows the
-    palette to have more hue variance across the range, which can be helpful
-    for distinguishing a wider range of values.
-
-    Requires IPython 2+ and must be used in the notebook.
-
-    Parameters
-    ----------
-    as_cmap : bool
-        If True, the return value is a matplotlib colormap rather than a
-        list of discrete colors.
-
-    Returns
-    -------
-    pal or cmap : list of colors or matplotlib colormap
-        Object that can be passed to plotting functions.
-
-    See Also
-    --------
-    cubehelix_palette : Create a sequential palette or colormap using the
-                        cubehelix system.
-
-    """
-    from IPython.html.widgets import (interact,
-                                      FloatSliderWidget, IntSliderWidget)
-
-    pal = []
-    if as_cmap:
-        cmap = _init_mutable_colormap()
-
-    @interact
-    def choose_cubehelix(n_colors=IntSliderWidget(min=2, max=16, value=9),
-                         start=FloatSliderWidget(min=0, max=3, value=0),
-                         rot=FloatSliderWidget(min=-1, max=1, value=.4),
-                         gamma=FloatSliderWidget(min=0, max=5, value=1),
-                         hue=FloatSliderWidget(min=0, max=1, value=.8),
-                         light=FloatSliderWidget(min=0, max=1, value=.85),
-                         dark=FloatSliderWidget(min=0, max=1, value=.15),
-                         reverse=False):
-
-        if as_cmap:
-            colors = cubehelix_palette(256, start, rot, gamma,
-                                       hue, light, dark, reverse)
-            _update_lut(cmap, np.c_[colors, np.ones(256)])
-            _show_cmap(cmap)
-        else:
-            pal[:] = cubehelix_palette(n_colors, start, rot, gamma,
-                                       hue, light, dark, reverse)
-            palplot(pal)
-
-    if as_cmap:
-        return cmap
-    return pal
